<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class supports Services
 *
 * @package    ADSL
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Model_Service_Plugin_Adsl extends Model_Service_Plugin {
	protected $_table_name = 'service__adsl';
	protected $_created_column = FALSE;
	protected $_updated_column = FALSE;

	// Relationships
	protected $_has_one = array(
		'provided_plan'=>array('model'=>'Product_Plugin_Adsl','far_key'=>'provided_adsl_plan_id','foreign_key'=>'id'),
	);

	protected $_has_many = array(
		'traffic'=>array('model'=>'Service_Plugin_Adsl_Traffic','foreign_key'=>'service','far_key'=>'service_username'),
	);

	protected $_display_filters = array(
		'service_connect_date'=>array(
			array('Config::date',array(':value')),
		),
		'service_contract_date'=>array(
			array('Config::date',array(':value')),
		),
	);

	protected $_save_message = TRUE;

	// Required abstract functions

	public function expire() {
		// We'll leave it to the Service record to determine when this service expires
		return NULL;
	}

	public function name() {
		return $this->service_number;
	}

	public function password() {
		return $this->service_password;
	}

	public function username() {
		return $this->service_username;
	}

	// Local functions

	/**
	 * If we override the plan that the customers gets (from what the supplier provides).
	 */
	public function admin_plan() {
		return $this->provided_adsl_plan_id ? $this->provided_plan : $this->service->product->plugin();
	}

	/**
	 * Calculate our contract start and end dates
	 */
	public function contract_date_start($format=FALSE) {
		return $format ? Config::date($this->service_contract_date) : $this->service_contract_date;
	}

	public function contract_date_end($format=FALSE) {
		// ADSL Contract Terms are held in the ADSL Plan
		$x = strtotime(sprintf('+%s months',$this->service->plugin()->contract_term),$this->service_contract_date);

		return $format ? Config::date($x) : $x;
	}

	/**
	 * This function will take an array of numbers and change it into a cumulative array
	 */
	private function cumulative($array) {
		$result = array();
		$s = 0;

		foreach ($array as $k => $v)
			$result[$k] = ($s += $v);

		return $result;
	}

	/**
	 * This function will return the months that have traffic data.
	 * This array can be used in a select list to display the traffic for that month
	 */
	public function get_traffic_months() {
		$x = array_keys($this->get_traffic_monthlytype());

		if (! $x)
			return array();

		$months = array_combine($x,$x);

		arsort($months);

		return $months;
	}

	/**
	 * Get daily traffic data, broken down by type
	 * @todo This needs to get the plan that was invoiced, not the current plan..
	 */
	public function get_traffic_dailytype($period) {
		$result = array();

		if (is_null($period))
			$period = strtotime('yesterday');

		$t = $this->traffic
			->where('date','>=',date('Y-m-d',mktime(0,0,0,date('m',$period),1,date('Y',$period))))
			->and_where('date','<=',date('Y-m-d',strtotime('last day of '.date('M Y',$period))));

		foreach ($t->find_all() as $to) {
			$day = date('j',strtotime($to->date));

			$result[$day] = $this->plan()->allowance($to->traffic_data());
		}

		return $result;
	}

	/**
	 * Get monthly traffic data, broken down by type
	 * @todo This needs to get the plan that was invoiced, not the current plan..
	 */
	public function get_traffic_monthlytype($period=NULL,$periodstart=NULL,$format=FALSE,$ceil=FALSE) {
		$result = array();

		if (is_null($period))
			$period = strtotime('yesterday');
		if (is_null($periodstart))
			$periodstart = mktime(0,0,0,date('m',$period),1,date('Y',$period)-1);

		$t = $this->traffic
			->select(array('date_format(date,\'%y-%m\')','month'))
			->selectsummetric()
			->and_where('date','>=',date('Y-m-d',$periodstart))
			->and_where('date','<=',date('Y-m-d',strtotime('last day of '.date('M Y',$period))))
			->group_by('date_format(date,\'%Y-%m\')');

		foreach ($t->find_all() as $to) {
			$index = $to->month;

			$result[$index] = $this->plan()->allowance($to->traffic_data(),$format,FALSE,$ceil);
		}

		return $result;
	}

	/**
	 * Get traffic type data, broken down by day
	 */
	public function get_traffic_typedaily($period=NULL) {
		$result = array();

		if (is_null($period))
			$period = strtotime('yesterday');

		foreach ($this->get_traffic_dailytype($period) as $day => $data)
			foreach ($data as $metric => $value)
				$result[$metric][$day] = $value;

		return $result;
	}

	/**
	 * Get traffic type data, broken down by month
	 */
	public function get_traffic_typemonthly($period=NULL,$ceil=FALSE) {
		$result = array();

		if (is_null($period))
			$period = strtotime('yesterday');

		foreach ($this->get_traffic_monthlytype($period,NULL,FALSE,$ceil) as $day => $data)
			foreach ($data as $metric => $value)
				$result[$metric][$day] = $value;

		return $result;
	}

	public function hasOffPeak() {
		return $this->plan()->hasOffPeak();
	}

	/**
	 * Return the IP Address for the service
	 */
	public function ipaddress() {
		return $this->ipaddress ? $this->ipaddress : _('Dynamic');
	}

	/**
	 * If we override the plan that the customers gets (from what the supplier provides).
	 * @todo This needs to get the plan that was invoiced, not the current plan..
	 */
	public function plan($month=NULL) {
		return is_null($month) ? $this->service->product->plugin() : $this->plandate($month);
	}

	/**
	 * For a particular month, select the PLAN that the user was on
	 */
	public function plandate($month) {
		throw new Kohana_Exception('This function hasnt been written yet.');
	}

	/**
	 * Return the template variables, used mainly in emailing
	 *
	 * @param array Variables that need to be expanded
	 * @param array Data that was previously calculated
	 */
	public function template_variables(array $array,array $data=array()) {
		$result = array();

		$friendly = array(
			'base_down_peak'=>'Peak',
			'base_down_offpeak'=>'OffPeak',
			'cumulative_base_down_peak'=>'Total Peak',
			'cumulative_base_down_offpeak'=>'Total OffPeak',
		);

		if (! isset($data['allow']))
			$data['allow'] = $this->plan()->allowance(array(),FALSE,TRUE);

		if (! isset($data['last']))
			$data['last'] = $this->traffic->find_last()->date();

		if (! isset($data['used']))
			$data['used'] = $this->traffic_month($data['last']);

		if (! isset($data['day']))
			$data['day'] = date('d',strtotime('yesterday'));

		$daysleft = date('d',strtotime('last day of',$data['last']))-$data['day'];
		$traffic_type = $this->get_traffic_typedaily($data['last']);

		$google = GoogleChart::factory('Legacy')
			->type('vertical_bar')
			->title(sprintf('DSL traffic usage as at %s',Config::date($data['last'])));

		foreach ($traffic_type as $k => $details)
			$google->sdata(array('yl'=>($x=$this->traffic->friendly($k))),array($x=>$details));

		// Work out comulative numbers
		foreach ($traffic_type as $k => $details)
			$google->sdata(array('yr'=>($x='Cumulative '.$this->traffic->friendly($k))),array($x=>$this->cumulative($traffic_type[$k])));

		foreach ($array as $item) {
			switch ($item) {
				case 'MONTH_GRAPH': $value = (string)$google; break;
				case 'MONTH_TABLE': $value = $google->table(FALSE,array(
					'table'=>'style="border: 1px solid #bebcb7; padding: 5px 5px; background: none repeat scroll 0% 0% #f8f7f5; font-size: 70%;"',
					)); break;

				case 'OFFPEAK_ALLOWANCE': $value = isset($data['allow']['base_down_offpeak']) ? $data['allow']['base_down_offpeak'].' MB' : '-'; break;
				case 'OFFPEAK_USAGE': $value = isset($data['used']['base_down_offpeak']) ? $data['used']['base_down_offpeak'].' MB' : '-'; break;

				case 'PEAK_ALLOWANCE': $value = isset($data['allow']['base_down_peak']) ? $data['allow']['base_down_peak'].' MB' : '-'; break;
				case 'PEAK_USAGE': $value = isset($data['used']['base_down_peak']) ? $data['used']['base_down_peak'].' MB' : '-'; break;

				case 'OFFPEAK_AVERAGE': $value = isset($data['used']['base_down_offpeak']) ? round($data['used']['base_down_offpeak']/$data['day'],2).' MB' : '-'; break;
				case 'OFFPEAK_AVERAGE_REMAIN': $value = ((isset($data['used']['base_down_offpeak']) AND ($data['allow']['base_down_offpeak'] > $data['used']['base_down_offpeak']) AND $daysleft) ? round(($data['allow']['base_down_offpeak']-$data['used']['base_down_offpeak'])/$daysleft,2).' MB' : '-'); break;

				case 'PEAK_AVERAGE': $value = isset($data['used']['base_down_peak']) ? round($data['used']['base_down_peak']/$data['day'],2).' MB' : '-'; break;
				case 'PEAK_AVERAGE_REMAIN': $value = ((isset($data['used']['base_down_peak']) AND ($data['allow']['base_down_peak'] > $data['used']['base_down_peak']) AND $daysleft) ? round(($data['allow']['base_down_peak']-$data['used']['base_down_peak'])/$daysleft,2).' MB' : '-'); break;

				case 'SERVICE_NUMBER': $value = $this->service_number; break;
				case 'USAGE_DATE': $value = Config::date($data['last']); break;
				case 'USER_NAME': $value = $this->service->account->name(); break;
				default:
					$value = '';
			}

			$result[$item] = $value;
		}

		return $result;
	}

	/**
	 * Calculate the Excess Traffic Charges
	 */
	public function traffic_excess($period=NULL,$charge=FALSE,$format=FALSE) {
		$result = array();

		if ($this->plan()->extra_charged) {
			if (is_null($period))
				$period = strtotime('yesterday');

			if ($x=$this->get_traffic_monthlytype(strtotime('last day of '.date('M Y',$period)),strtotime('first day of '.date('M Y',$period)))) {

				$c = $this->plan()->cost_extra();
				foreach ($this->plan()->allowance(array_pop($x),FALSE,TRUE,TRUE) as $k=>$v)
					if (isset($c[$k]) AND $v > 0) {
						$result[$k] = $charge ? $c[$k]*$v : $v;

						if ($charge AND $format)
							$result[$k] = Currency::display(Tax::add($result[$k]));
					}
			}
		}

		if (! $result AND $charge AND $format)
			$result = array(Currency::display(0));

		return $format ? join('/',$result) : $result;
	}

	/**
	 * Render a chart of traffic
	 */
	public function traffic_graph($month=NULL) {
		$highchart = HighChart::factory('Combo');

		switch ($this->plan()->metric) {
			case '1000' : $highchart->ymetric('GB'); break;
			case '1' : $highchart->ymetric('MB'); break;
			default: $highchart->ymetric('?');
		}

		$c=0;
		// If we came in via a post to show a particular month, then show that, otherwise show the yearly result
		if (! is_null($month) AND trim($month)) {
			$highchart->title(sprintf('DSL traffic usage for %s',$_POST['month']));
			$x = $this->get_traffic_typedaily(strtotime($_POST['month'].'-01'));

		} else {
			$highchart->title(sprintf('Monthly DSL traffic usage as at %s',$this->traffic->find_last()->date));
			$x = $this->get_traffic_typemonthly(NULL,TRUE);
		}

		foreach ($x as $k => $details) {
			$highchart->series('column','yl')
				->name($this->traffic->friendly($k))
				->data($x[$k])
				->index($c)
				->order($c++*-1); // This is a kludge to get around highcharts rendering from the bottom up.

			$highchart->autopie($k);
		}


		return (string)$highchart;
	}

	/**
	 * Get the traffic for a month
	 */
	public function traffic_month($period,$format=FALSE,$ceil=FALSE) {
		$x = $this->get_traffic_monthlytype(strtotime('last day of '.date('M Y',$period)),strtotime('first day of '.date('M Y',$period)),$format,$ceil);

		return $x ? array_pop($x) : 0;
	}

	/**
	 * Determine if we alert traffic
	 *
	 * We alert traffic if:
	 * + 80% of usage every 3 days
	 * + average daily usage > allowance every 5 days
	 * + last day of the period
	 *
	 * @return bool
	 */
	public function traffic_report() {
		$result = array();

		$result['day'] = date('d',strtotime('yesterday'));
		$result['last'] = $this->traffic->find_last()->date();

		if (! Period::inMonth($result['last'],strtotime('yesterday')))
			return array();

		$lastday = date('d',strtotime('last day of',$result['last']));

		$result['allow'] = $this->plan()->allowance(array(),FALSE,TRUE);
		$result['used'] = $this->traffic_month($result['last']);

		if (! array_sum($result['used']))
			return array();

		// If we are the last day of the period, and we had traffic
		if ($result['day'] == $lastday)
			return $result;

		foreach ($result['used'] as $k => $v) {
			// If we are at 80% usage
			// @todo This should be a setup config item
			if ($v/($result['allow'][$k] > 0 ? $result['allow'][$k] : 1) >= .8 AND $result['day']%3 == 0)
				return $result;

			// If our average is greater than our allowance
			// @todo This should be a setup config item
			if ($result['day']%5 == 0 AND ($v/$result['day'] > $result['allow'][$k]/$result['day']))
				return $result;
		}

		// If we get here, then we dont need to report usage.
		return array();
	}

	/**
	 * Render a table of traffic
	 */
	public function traffic_table($month=NULL) {
		// If we came in via a post to show a particular month, then show that, otherwise show the yearly result
		if (! is_null($month) AND trim($month)) {
			$x = $this->get_traffic_dailytype(strtotime($_POST['month'].'-01'));
			$index = 'Date';

		} else {
			$x = $this->get_traffic_monthlytype(NULL,NULL,FALSE,TRUE);
			$index = 'Month';
		}

		return View::factory(sprintf('service/user/plugin/%s/table_traffic',$this->plugin()))
			->set('index',$index)
			->set('th',array_keys($this->plan()->allowance()))
			->set('td',$x)
			->set('o',$this->traffic);
	}

	/**
	 * Search for services matching a term
	 */
	public function list_autocomplete($term,$index,$value,array $label,array $limit=array(),array $options=NULL) {
		$ao = Auth::instance()->get_user();

		$options['key'] = 'id';
		$options['object'] = DB::select($this->_table_name.'.id',$this->_table_name.'.service_number')
			->from($this->_table_name)
			->join('service')
			->on('service.id','=',$this->_table_name.'.service_id')
			->where('service.account_id','IN',$ao->RTM->customers($ao->RTM))
			->where_open()
			->and_where($this->_table_name.'.service_number','like','%'.$term.'%')
			->or_where($this->_table_name.'.service_address','like','%'.$term.'%')
			->or_where($this->_table_name.'.ipaddress','like','%'.$term.'%')
			->where_close();

		return parent::list_autocomplete($term,$index,$value,$label,$limit,$options);
	}

	/**
	 * Get specific service details for use in other modules
	 * For Example: Invoice
	 *
	 * @todo Make the rendered items configurable
	 * @todo Change this method name, now that it is public
	 */
	public function _details($type) {
		switch ($type) {
			case 'invoice_detail_items':
				return array(
					_('Service Address')=>$this->service_address ? $this->display('service_address') : '>NotSet<',
					_('Contract Until')=>$this->contract_date_end(TRUE),
				);
				break;
			default:
				return parent::$_details($type);
		}
	}
}
?>
